@@ -218,7 +218,7 @@ struct _GstRTSPClientSink {
   gboolean         streams_collected;
 
   /* TRUE when streams have been blocked */
-  gboolean         streams_blocked;
+  guint            n_streams_blocked;
   GMutex           block_streams_lock;
   GCond            block_streams_cond;
 
