@@ -485,15 +485,15 @@ test_setup_response_200_multicast (GstRTSPClient * client,
 
   fail_unless (expected_transport != NULL);
 
-  fail_unless (gst_rtsp_message_get_type (response) ==
+  fail_unless_equals_int (gst_rtsp_message_get_type (response),
       GST_RTSP_MESSAGE_RESPONSE);
 
   fail_unless (gst_rtsp_message_parse_response (response, &code, &reason,
           &version)
       == GST_RTSP_OK);
-  fail_unless (code == GST_RTSP_STS_OK);
-  fail_unless (g_str_equal (reason, "OK"));
-  fail_unless (version == GST_RTSP_VERSION_1_0);
+  fail_unless_equals_int (code, GST_RTSP_STS_OK);
+  fail_unless_equals_string (reason, "OK");
+  fail_unless_equals_int (version, GST_RTSP_VERSION_1_0);
 
   fail_unless (gst_rtsp_message_get_header (response, GST_RTSP_HDR_CSEQ, &str,
           0) == GST_RTSP_OK);
@@ -502,7 +502,7 @@ test_setup_response_200_multicast (GstRTSPClient * client,
   fail_unless (gst_rtsp_message_get_header (response, GST_RTSP_HDR_TRANSPORT,
           &str, 0) == GST_RTSP_OK);
 
-  fail_unless (!strcmp (str, expected_transport));
+  fail_unless_equals_string (str, expected_transport);
 
   fail_unless (gst_rtsp_message_get_header (response, GST_RTSP_HDR_SESSION,
           &str, 0) == GST_RTSP_OK);
@@ -793,7 +793,6 @@ GST_START_TEST (test_client_multicast_invalid_transport_specific)
   g_object_unref (session_pool);
 
 
-
   /* simple SETUP with a valid URI and multicast, but an invalid prt */
   fail_unless (gst_rtsp_message_init_request (&request, GST_RTSP_SETUP,
           "rtsp://localhost/test/stream=0") == GST_RTSP_OK);
@@ -813,7 +812,6 @@ GST_START_TEST (test_client_multicast_invalid_transport_specific)
   g_object_unref (session_pool);
 
 
-
   /* simple SETUP with a valid URI and multicast, but an invalid ttl */
   fail_unless (gst_rtsp_message_init_request (&request, GST_RTSP_SETUP,
           "rtsp://localhost/test/stream=0") == GST_RTSP_OK);
@@ -832,7 +830,6 @@ GST_START_TEST (test_client_multicast_invalid_transport_specific)
   fail_unless (gst_rtsp_session_pool_get_n_sessions (session_pool) == 0);
   g_object_unref (session_pool);
 
-
   teardown_client (client);
   g_object_unref (ctx.auth);
   gst_rtsp_token_unref (ctx.token);
